@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf8 -*-
 # ============================================================================
-#  Copyright (c) 2013-2019 nexB Inc. http://www.nexb.com/ - All rights reserved.
+#  Copyright (c) nexB Inc. http://www.nexb.com/ - All rights reserved.
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
 #  You may obtain a copy of the License at
@@ -13,48 +13,33 @@
 #  limitations under the License.
 # ============================================================================
 
-from __future__ import absolute_import
-from __future__ import print_function
-from __future__ import unicode_literals
+from collections import OrderedDict
 
 import codecs
-from collections import OrderedDict
+import csv
 import json
 import ntpath
+import openpyxl
 import os
 import posixpath
 import re
 import shutil
 import string
 import sys
+from distutils.dir_util import copy_tree
+from itertools import zip_longest
 
 from attributecode import CRITICAL
 from attributecode import WARNING
 from attributecode import Error
 
-
-python2 = sys.version_info[0] < 3
-
-if python2:  # pragma: nocover
-    from itertools import izip_longest as zip_longest  # NOQA
-else:  # pragma: nocover
-    from itertools import zip_longest  # NOQA
-
-if python2:  # pragma: nocover
-    from backports import csv  # NOQA
-    # monkey patch backports.csv until bug is fixed
-    # https://github.com/ryanhiebert/backports.csv/issues/30
-    csv.dict = OrderedDict
-else:  # pragma: nocover
-    import csv  # NOQA
-
-
 on_windows = 'win32' in sys.platform
 
 # boolean field name
 boolean_fields = ['redistribute', 'attribute', 'track_change', 'modified', 'internal_use_only']
 file_fields = ['about_resource', 'notice_file', 'changelog_file', 'author_file']
 
+
 def to_posix(path):
     """
     Return a path using the posix path separator given a path that may contain
@@ -70,7 +55,7 @@ UNC_PREFIX = u'\\\\?\\'
 UNC_PREFIX_POSIX = to_posix(UNC_PREFIX)
 UNC_PREFIXES = (UNC_PREFIX_POSIX, UNC_PREFIX,)
 
-valid_file_chars = string.digits + string.ascii_letters + '_-.+()~[]{}|' + ' '
+valid_file_chars = string.digits + string.ascii_letters + '_-.+()~[]{}|@%' + ' '
 
 
 def invalid_chars(path):
@@ -126,6 +111,7 @@ def check_file_names(paths):
             seen[path] = orig_path
     return errors
 
+
 def wrap_boolean_value(context):
     updated_context = ''
     for line in context.splitlines():
@@ -141,6 +127,7 @@ def wrap_boolean_value(context):
             updated_context += line + '\n'
     return updated_context
 
+
 def replace_tab_with_spaces(context):
     updated_context = ''
     for line in context.splitlines():
@@ -150,6 +137,7 @@ def replace_tab_with_spaces(context):
         updated_context += line.replace('\t', '    ') + '\n'
     return updated_context
 
+
 # TODO: rename to normalize_path
 def get_absolute(location):
     """
@@ -192,20 +180,24 @@ def get_about_locations(location):
             yield loc
 
 
+def norm(p):
+    """
+    Normalize the path
+    """
+    if p.startswith(UNC_PREFIX) or p.startswith(to_posix(UNC_PREFIX)):
+        p = p.strip(UNC_PREFIX).strip(to_posix(UNC_PREFIX))
+    p = to_posix(p)
+    p = p.strip(posixpath.sep)
+    p = posixpath.normpath(p)
+    return p
+
+
 def get_relative_path(base_loc, full_loc):
     """
     Return a posix path for a given full location relative to a base location.
     The first segment of the different between full_loc and base_loc will become
     the first segment of the returned path.
     """
-    def norm(p):
-        if p.startswith(UNC_PREFIX) or p.startswith(to_posix(UNC_PREFIX)):
-            p = p.strip(UNC_PREFIX).strip(to_posix(UNC_PREFIX))
-        p = to_posix(p)
-        p = p.strip(posixpath.sep)
-        p = posixpath.normpath(p)
-        return p
-
     base = norm(base_loc)
     path = norm(full_loc)
 
@@ -264,76 +256,28 @@ def resource_name(path):
     return right.strip()
 
 
-
 def load_csv(location):
     """
     Read CSV at `location`, return a list of ordered dictionaries, one
     for each row.
     """
     results = []
-    # FIXME: why ignore encoding errors here?
     with codecs.open(location, mode='rb', encoding='utf-8-sig',
-                     errors='ignore') as csvfile:
+                     errors='replace') as csvfile:
         for row in csv.DictReader(csvfile):
             # convert all the column keys to lower case
-            updated_row = OrderedDict(
-                [(key.lower(), value) for key, value in row.items()]
-            )
+            updated_row = {key.lower().strip(): value for key, value in row.items()}
             results.append(updated_row)
     return results
 
-
 def load_json(location):
     """
     Read JSON file at `location` and return a list of ordered dicts, one for
     each entry.
     """
-    # FIXME: IMHO we should know where the JSON is from and its shape
-    # FIXME use: object_pairs_hook=OrderedDict
     with open(location) as json_file:
         results = json.load(json_file)
 
-    # If the loaded JSON is not a list,
-    # - JSON output from AboutCode Manager:
-    # look for the "components" field as it is the field
-    # that contain everything the tool needs and ignore other fields.
-    # For instance,
-    # {
-    #    "aboutcode_manager_notice":"xyz",
-    #    "aboutcode_manager_version":"xxx",
-    #    "components":
-    #    [{
-    #        "license_expression":"apache-2.0",
-    #        "copyright":"Copyright (c) 2017 nexB Inc.",
-    #        "path":"ScanCode",
-    #        ...
-    #    }]
-    # }
-    #
-    # - JSON output from ScanCode:
-    # look for the "files" field as it is the field
-    # that contain everything the tool needs and ignore other fields:
-    # For instance,
-    # {
-    #    "scancode_notice":"xyz",
-    #    "scancode_version":"xxx",
-    #    "files":
-    #    [{
-    #        "path": "test",
-    #        "type": "directory",
-    #        "name": "test",
-    #        ...
-    #    }]
-    # }
-    #
-    # - JSON file that is not produced by scancode or aboutcode toolkit
-    # For instance,
-    # {
-    #    "path": "test",
-    #    "type": "directory",
-    #    "name": "test",
-    #    ...
-    # }
     # FIXME: this is too clever and complex... IMHO we should not try to guess the format.
     # instead a command line option should be provided explictly to say what is the format
     if isinstance(results, list):
@@ -354,10 +298,7 @@ def have_network_connection():
     Return True if an HTTP connection to some public web site is possible.
     """
     import socket
-    if python2:
-        import httplib  # NOQA
-    else:
-        import http.client as httplib  # NOQA
+    import http.client as httplib
 
     http_connection = httplib.HTTPConnection('dejacode.org', timeout=10)  # NOQA
     try:
@@ -433,15 +374,16 @@ def copy_license_notice_files(fields, base_dir, reference_dir, afp):
     about_file_path value, this function will copy to the base_dir the
     license_file or notice_file if found in the reference_dir
     """
+    errors = []
     copy_file_name = ''
     for key, value in fields:
         if key == 'license_file' or key == 'notice_file':
             if value:
                 # This is to handle multiple license_file value in CSV format
                 # The following code will construct a list to contain the
-                # license file(s) that need to be copied. 
+                # license file(s) that need to be copied.
                 # Note that *ONLY* license_file field allows \n. Others file
-                # fields that have \n will prompts error at validation stage 
+                # fields that have \n will prompts error at validation stage
                 file_list = []
                 if '\n' in value:
                     f_list = value.split('\n')
@@ -466,26 +408,57 @@ def copy_license_notice_files(fields, base_dir, reference_dir, afp):
                 from_lic_path = posixpath.join(to_posix(reference_dir), copy_file_name)
                 about_file_dir = os.path.dirname(to_posix(afp)).lstrip('/')
                 to_lic_path = posixpath.join(to_posix(base_dir), about_file_dir)
-    
-                if on_windows:
-                    from_lic_path = add_unc(from_lic_path)
-                    to_lic_path = add_unc(to_lic_path)
-    
-                # Strip the white spaces
-                from_lic_path = from_lic_path.strip()
-                to_lic_path = to_lic_path.strip()
-    
-                # Errors will be captured when doing the validation
-                if not posixpath.exists(from_lic_path):
-                    continue
-    
-                if not posixpath.exists(to_lic_path):
-                    os.makedirs(to_lic_path)
-                try:
-                    shutil.copy2(from_lic_path, to_lic_path)
-                except Exception as e:
-                    print(repr(e))
-                    print('Cannot copy file at %(from_lic_path)r.' % locals())
+                if not os.path.exists(posixpath.join(to_lic_path, copy_file_name)):
+                    err = copy_file(from_lic_path, to_lic_path)
+                    if err:
+                        errors.append(err)
+    return errors
+
+
+def copy_file(from_path, to_path):
+    error = ''
+    # Return if the from_path is empty or None.
+    if not from_path:
+        return
+
+    if on_windows:
+        if not from_path.startswith(UNC_PREFIXES):
+            from_path = add_unc(from_path)
+        if not to_path.startswith(UNC_PREFIXES):
+            to_path = add_unc(to_path)
+
+    # Strip the white spaces
+    from_path = from_path.strip()
+    to_path = to_path.strip()
+    # Errors will be captured when doing the validation
+    if not os.path.exists(from_path):
+        return ''
+
+    if not posixpath.exists(to_path):
+        os.makedirs(to_path)
+    try:
+        if os.path.isdir(from_path):
+            # Copy the whole directory structure
+            if from_path.endswith('/'):
+                from_path = from_path.rpartition('/')[0]
+            folder_name = os.path.basename(from_path)
+            to_path = os.path.join(to_path, folder_name)
+            if os.path.exists(to_path):
+                msg = to_path + ' is already existed and is replaced by ' + from_path
+                error = Error(WARNING, msg)
+            copy_tree(from_path, to_path)
+        else:
+            file_name = os.path.basename(from_path)
+            to_file_path = os.path.join(to_path, file_name)
+            if os.path.exists(to_file_path):
+                msg = to_file_path + ' is already existed and is replaced by ' + from_path
+                error = Error(WARNING, msg)
+            shutil.copy2(from_path, to_path)
+        return error
+    except Exception as e:
+        msg = 'Cannot copy file at %(from_path)r.' % locals()
+        error = Error(CRITICAL, msg)
+        return error
 
 
 # FIXME: we should use a license object instead
@@ -497,6 +470,8 @@ def ungroup_licenses(licenses):
     lic_name = []
     lic_file = []
     lic_url = []
+    spdx_lic_key = []
+    lic_score = []
     for lic in licenses:
         if 'key' in lic:
             lic_key.append(lic['key'])
@@ -506,14 +481,18 @@ def ungroup_licenses(licenses):
             lic_file.append(lic['file'])
         if 'url' in lic:
             lic_url.append(lic['url'])
-    return lic_key, lic_name, lic_file, lic_url
+        if 'spdx_license_key' in lic:
+            spdx_lic_key.append(lic['spdx_license_key'])
+        if 'score' in lic:
+            lic_score.append(lic['score'])
+    return lic_key, lic_name, lic_file, lic_url, spdx_lic_key, lic_score
 
 
 # FIXME: add docstring
-def format_about_dict_for_csv_output(about_dictionary_list):
-    csv_formatted_list = []
+def format_about_dict_output(about_dictionary_list):
+    formatted_list = []
     for element in about_dictionary_list:
-        row_list = OrderedDict()
+        row_list = dict()
         for key in element:
             if element[key]:
                 if isinstance(element[key], list):
@@ -522,8 +501,8 @@ def format_about_dict_for_csv_output(about_dictionary_list):
                     row_list[key] = u'\n'.join((element[key].keys()))
                 else:
                     row_list[key] = element[key]
-        csv_formatted_list.append(row_list)
-    return csv_formatted_list
+        formatted_list.append(row_list)
+    return formatted_list
 
 
 # FIXME: add docstring
@@ -531,7 +510,7 @@ def format_about_dict_for_json_output(about_dictionary_list):
     licenses = ['license_key', 'license_name', 'license_file', 'license_url']
     json_formatted_list = []
     for element in about_dictionary_list:
-        row_list = OrderedDict()
+        row_list = dict()
         # FIXME: aboid using parallel list... use an object instead
         license_key = []
         license_name = []
@@ -560,7 +539,7 @@ def format_about_dict_for_json_output(about_dictionary_list):
         if license_group:
             licenses_list = []
             for lic_group in license_group:
-                lic_dict = OrderedDict()
+                lic_dict = dict()
                 if lic_group[0]:
                     lic_dict['key'] = lic_group[0]
                 if lic_group[1]:
@@ -595,7 +574,159 @@ def filter_errors(errors, minimum_severity=WARNING):
     Return a list of unique `errors` Error object filtering errors that have a
     severity below `minimum_severity`.
     """
-    return unique([e for e in errors if e.severity >= minimum_severity])
+    return [e for e in errors if e.severity >= minimum_severity]
+
+
+def create_dir(location):
+    """
+    Create directory or directory tree at location, ensuring it is readable
+    and writeable.
+    """
+    import stat
+    if not os.path.exists(location):
+        os.makedirs(location)
+        os.chmod(location, stat.S_IRWXU | stat.S_IRWXG
+                 | stat.S_IROTH | stat.S_IXOTH)
+
+
+def get_temp_dir(sub_dir_path=None):
+    """
+    Create a unique new temporary directory location. Create directories
+    identified by sub_dir_path if provided in this temporary directory.
+    Return the location for this unique directory joined with the
+    sub_dir_path if any.
+    """
+    new_temp_dir = build_temp_dir()
+
+    if sub_dir_path:
+        # create a sub directory hierarchy if requested
+        new_temp_dir = os.path.join(new_temp_dir, sub_dir_path)
+        create_dir(new_temp_dir)
+    return new_temp_dir
+
+
+def build_temp_dir(prefix='attributecode-'):
+    """
+    Create and return a new unique empty directory created in base_dir.
+    """
+    import tempfile
+    location = tempfile.mkdtemp(prefix=prefix)
+    create_dir(location)
+    return location
+
+def get_file_text(file_name, reference):
+    """
+    Return the file content from the license_file/notice_file field from the
+    given reference directory.
+    """
+    error = ''
+    text = ''
+    file_path = os.path.join(reference, file_name)
+    if not os.path.exists(file_path):
+        msg = "The file " + file_path + " does not exist"
+        error = Error(CRITICAL, msg)
+    else:
+        with codecs.open(file_path, 'rb', encoding='utf-8-sig', errors='replace') as txt:
+        #with io.open(file_path, encoding='utf-8') as txt:
+            text = txt.read()
+    return error, text
+
+def convert_object_to_dict(about):
+    """
+    Convert the list of field object
+        [Field(name='name', value=''), Field(name='version', value='')]
+    to a dictionary
+    """
+    about_dict = {}
+    # Convert all the supported fields into a dictionary
+    fields_dict = getattr(about, 'fields')
+    custom_fields_dict = getattr(about, 'custom_fields')
+    supported_dict = {**fields_dict, **custom_fields_dict}
+    for field in supported_dict:
+        key = supported_dict[field].name
+        value = supported_dict[field].value
+        about_dict[key] = value
+    return about_dict
+
+def load_scancode_json(location):
+    """
+    Read the scancode JSON file at `location` and return a list of dictionaries.
+    """
+    updated_results = []
+
+    with open(location) as json_file:
+        results = json.load(json_file)
+    results = results['files']
+    # Rename the "path" to "about_resource"
+    for item in results:
+        updated_dict = {}
+        for key in item:
+            if key == 'path':
+                updated_dict['about_resource'] = item[key]
+            else:
+                updated_dict[key] = item[key]
+        updated_results.append(updated_dict)
+    return updated_results
+
+def load_excel(location):
+    """
+    Read XLSX at `location`, return a list of ordered dictionaries, one
+    for each row.
+    """
+    results = []
+    errors = []
+    import warnings
+
+    # This is to prevent showing the: warn("Workbook contains no default style, apply openpyxl's default")
+    with warnings.catch_warnings(record=True):
+        sheet_obj = openpyxl.load_workbook(location).active
+    max_col = sheet_obj.max_column
+
+    index = 1
+    col_keys = []
+    mapping_dict = {}
+
+    while index <= max_col:
+        value = sheet_obj.cell(row=1, column=index).value
+        if value in col_keys:
+            msg = 'Duplicated column name, ' + str(value) + ', detected.' 
+            errors.append(Error(CRITICAL, msg))
+            return errors, results
+        if value in mapping_dict:
+            value = mapping_dict[value]
+        col_keys.append(value)
+        index = index + 1
+
+    for row in sheet_obj.iter_rows(min_row=2, values_only=True):
+        row_dict = OrderedDict()
+        index = 0
+        while index < max_col:
+            value = row[index]
+            if value:
+                row_dict[col_keys[index]] = value
+            else:
+                row_dict[col_keys[index]] = ''
+            index = index + 1
+        results.append(row_dict)
+    return errors, results
+
+def write_licenses(lic_dict, location):
+    import io
+
+    loc = to_posix(location)
+    errors = []
+
+    if not posixpath.exists(loc):
+        os.makedirs(add_unc(loc))
+    try:
+        for lic in lic_dict:
+            output_location = posixpath.join(loc, lic)
+            with io.open(output_location, 'w', encoding='utf-8', errors='replace') as out:
+                out.write(lic_dict[lic])
+    except Exception as e:
+        msg = str(e)
+        errors.append(Error(CRITICAL, msg))
+    return errors
 
 
 """
